/*
 * Nextcloud Android Library
 *
 * SPDX-FileCopyrightText: 2017-2024 Nextcloud GmbH and Nextcloud contributors
 * SPDX-FileCopyrightText: 2017 Andy Scherzinger <info@andy-scherzinger.de>
 * SPDX-FileCopyrightText: 2017 Mario Danic <mario@lovelyhq.com>
 * SPDX-License-Identifier: MIT
 */
package com.owncloud.android.lib.resources.notifications;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.reflect.TypeToken;
import com.nextcloud.common.NextcloudClient;
import com.nextcloud.operations.GetMethod;
import com.owncloud.android.lib.common.operations.RemoteOperation;
import com.owncloud.android.lib.common.operations.RemoteOperationResult;
import com.owncloud.android.lib.common.utils.Log_OC;
import com.owncloud.android.lib.resources.notifications.models.Notification;

import java.lang.reflect.Type;
import java.util.List;

/**
 * Provides the remote notifications from the server handling the following data structure
 * accessible via the notifications endpoint at {@value OCS_ROUTE_LIST_V12_AND_UP}, specified at
 * {@link "https://github.com/nextcloud/notifications/blob/master/docs/ocs-endpoint-v2.md"}.
 */
public class GetNotificationsRemoteOperation extends RemoteOperation<List<Notification>> {

    // OCS Route
    private static final String OCS_ROUTE_LIST_V12_AND_UP =
            "/ocs/v2.php/apps/notifications/api/v2/notifications" + JSON_FORMAT;

    private static final String TAG = GetNotificationsRemoteOperation.class.getSimpleName();

    // JSON Node names
    private static final String NODE_OCS = "ocs";

    private static final String NODE_DATA = "data";

    @Override
    public RemoteOperationResult<List<Notification>> run(NextcloudClient client) {
        RemoteOperationResult<List<Notification>> result;
        int status;
        GetMethod get = null;
        List<Notification> notifications;
        String url = client.getBaseUri() + OCS_ROUTE_LIST_V12_AND_UP;

        // get the notifications
        try {
            get = new GetMethod(url, true);

            status = client.execute(get);
            String response = get.getResponseBodyAsString();

            if (get.isSuccess()) {
                result = new RemoteOperationResult<>(true, get);
                Log_OC.d(TAG, "Successful response: " + response);

                // Parse the response
                notifications = parseResult(response);
                result.setResultData(notifications);
            } else {
                result = new RemoteOperationResult<>(false, get);
                Log_OC.e(TAG, "Failed response while getting user notifications ");
                Log_OC.e(TAG, "*** status code: " + status + " ; response message: " + response);
            }
        } catch (Exception e) {
            result = new RemoteOperationResult<>(e);
            Log_OC.e(TAG, "Exception while getting remote notifications", e);
        } finally {
            if (get != null) {
                get.releaseConnection();
            }
        }

        return result;
    }

    private List<Notification> parseResult(String response) {
        JsonObject jo = (JsonObject) JsonParser.parseString(response);
        JsonArray jsonDataArray = jo.getAsJsonObject(NODE_OCS).getAsJsonArray(NODE_DATA);

        Gson gson = new Gson();
        Type listType = new TypeToken<List<Notification>>() {
        }.getType();

        return gson.fromJson(jsonDataArray, listType);
    }
}
