/*
 * Nextcloud Android Library
 *
 * SPDX-FileCopyrightText: 2018-2024 Nextcloud GmbH and Nextcloud contributors
 * SPDX-FileCopyrightText: 2022 Álvaro Brey <alvaro@alvarobrey.com>
 * SPDX-FileCopyrightText: 2018-2020 Tobias Kaminsky <tobias@kaminsky.me>
 * SPDX-FileCopyrightText: 2019 Andy Scherzinger <info@andy-scherzinger.de>
 * SPDX-FileCopyrightText: 2019 Chris Narkiewicz <hello@ezaquarii.com>
 * SPDX-FileCopyrightText: 2015 ownCloud Inc.
 * SPDX-FileCopyrightText: 2014 David A. Velasco <dvelasco@solidgear.es>
 * SPDX-FileCopyrightText: 2014 masensio <masensio@solidgear.es>
 * SPDX-FileCopyrightText: 2012 Bartosz Przybylski <bart.p.pl@gmail.com>
 * SPDX-License-Identifier: MIT
 */
package com.owncloud.android.lib.resources.status;

import android.os.Parcel;
import android.os.Parcelable;

import androidx.annotation.NonNull;

public class OwnCloudVersion implements Comparable<OwnCloudVersion>, Parcelable {
    public static final OwnCloudVersion nextcloud_18 = new OwnCloudVersion(0x12000000); // 18.0
    public static final OwnCloudVersion nextcloud_19 = new OwnCloudVersion(0x13000000); // 19.0
    public static final OwnCloudVersion nextcloud_20 = new OwnCloudVersion(0x14000000); // 20.0

    private static final int MAX_DOTS = 3;

    // format is in version
    // 0xAABBCCDD
    // for version AA.BB.CC.DD
    // ie version 2.0.3 will be stored as 0x02000300
    private int version;
    private boolean versionValid;

    protected OwnCloudVersion(int version) {
        this.version = version;
        versionValid = true;
    }

    public OwnCloudVersion(String version) {
        this.version = 0;
        versionValid = false;
        int countDots = version.length() - version.replace(".", "").length();

        // Complete the version. Version must have 3 dots
        StringBuilder versionWithDots = new StringBuilder(version);
        for (int i = countDots; i < MAX_DOTS; i++) {
            versionWithDots.append(".0");
        }

        parseVersion(versionWithDots.toString());
    }

    @NonNull
    @Override
    public String toString() {
    	String versionToString = String.valueOf((version >> (8*MAX_DOTS)) % 256);
    	for (int i = MAX_DOTS - 1; i >= 0; i-- ) {
            versionToString = versionToString + "." + (version >> (8 * i)) % 256;
        }
        return versionToString;
    }
    
    public String getVersion() {
    	return toString();
    }

    @Override
    public int compareTo(@NonNull OwnCloudVersion another) {
        return Integer.compare(version, another.version);
    }

    public boolean isNewerOrEqual(@NonNull OwnCloudVersion another) {
        return version >= another.version;
    }

    public boolean isOlderThan(@NonNull OwnCloudVersion another) {
        return version < another.version;
    }

    public boolean isSameMajorVersion(@NonNull OwnCloudVersion another) {
        return this.getMajorVersionNumber() == another.getMajorVersionNumber();
    }

    public int getMajorVersionNumber() {
        return version >> (8 * MAX_DOTS) % 256;
    }

    private void parseVersion(String version) {
        try {
            this.version = getParsedVersion(version);
            versionValid = true;

        } catch (Exception e) {
            versionValid = false;
        }
    }

    static protected int getParsedVersion(String version) throws NumberFormatException {
        int versionValue = 0;

        // get only numeric part 
        version = version.replaceAll("[^\\d.]", "");

        String[] nums = version.split("\\.");
        for (int i = 0; i < nums.length && i <= MAX_DOTS; i++) {
            versionValue += Integer.parseInt(nums[i]);
            if (i < nums.length - 1) {
                versionValue = versionValue << 8;
            }
        }

        return versionValue;
    }

    /*
     * Autogenerated Parcelable
     */

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeInt(version);
        dest.writeByte(versionValid ? (byte) 1 : (byte) 0);
    }

    protected OwnCloudVersion(Parcel in) {
        version = in.readInt();
        versionValid = in.readByte() != 0;
    }

    public static final Creator<OwnCloudVersion> CREATOR = new Creator<OwnCloudVersion>() {
        @Override
        public OwnCloudVersion createFromParcel(Parcel source) {
            return new OwnCloudVersion(source);
        }

        @Override
        public OwnCloudVersion[] newArray(int size) {
            return new OwnCloudVersion[size];
        }
    };

    public boolean equals(final Object o) {
        if (o == this) return true;
        if (!(o instanceof OwnCloudVersion)) return false;
        final OwnCloudVersion other = (OwnCloudVersion) o;
        if (this.version != other.version) return false;
        return this.versionValid == other.versionValid;
    }

    public int hashCode() {
        final int PRIME = 59;
        int result = 1;
        result = result * PRIME + this.version;
        return result * PRIME + (this.versionValid ? 79 : 97);
    }

    public boolean isVersionValid() {
        return this.versionValid;
    }
}
